////////////////////////////////////////////////////////////
//Test SL015B-3
//AVR C source code
//ATmega16(7.3728MHz) + EWAVR 5.20
////////////////////////////////////////////////////////////   
#include <ioavr.h>
#include <intrinsics.h>
#include <avr_macros.h>
#include <iomacro.h>
#include <string.h>

//============================================
//  oscillator define
//============================================
#define OSC 7372800L

//============================================
//  Uart bound define
//============================================
#define  BOUND4800         4800
#define  BOUND9600         9600
#define  BOUND14400        14400
#define  BOUND19200        19200
#define  BOUND28800        28800
#define  BOUND38400        38400
#define  BOUND57600        57600
#define  BOUND115200       115200

//============================================
//  Pin define
//============================================
#define CARDIN PIND_Bit2

//============================================
// Uart Status define
//============================================
#define UARTSTATUS_FREE   0
#define UARTSTATUS_TX     1
#define UARTSTATUS_TXSUCC 2
#define UARTSTATUS_RXSUCC 3
#define UARTSTATUS_RXERR  4

//============================================
//  global variable define
//============================================
unsigned char g_cCardType;
unsigned long int g_iTimer;
unsigned char g_bOverTime;
unsigned char g_cUartTxCnt;
unsigned char g_cUartTxDataLen;
unsigned char *g_pUartTxDat;
unsigned char g_cUartTxCheckSum;
unsigned char g_cUartRxCnt;
unsigned char g_cUartRxBuf[60];
unsigned char g_cUartStatus;

//============================================
//  Command List, preamble + length + command 
//============================================
const unsigned char GetCardInformation[]        ={0xBA,0x02,0x31};       
const unsigned char GetBlcok0SecurityStatus[]   ={0xBA,0x04,0x32,0x0,0x01};
const unsigned char ReadBlocks[]                ={0xBA,0x04,0x33,0x0,0x10};                                     
const unsigned char WriteBlock0[]               ={0xBA,0x07,0x34,0x0,0x11,0x22,0x33,0x44};                                
const unsigned char WriteAFI[]                  ={0xBA,0x03,0x35,0xAA};  
const unsigned char WriteDSFID[]                ={0xBA,0x03,0x36,0xBB};                                
const unsigned char LockBlock0[]                ={0xBA,0x03,0x37,0x0};                                
const unsigned char LockAFI[]                   ={0xBA,0x02,0x38};                                          
const unsigned char LockDSFID[]                 ={0xBA,0x02,0x39};    
const unsigned char TurnOnRed[]                 ={0xBA,0x04,0x40,0x01,0x00};     
const unsigned char TurnOffRed[]                ={0xBA,0x04,0x40,0x01,0x01};      

//============================================
//  procedure define
//============================================
void Init_Hardware();
void Start_Time(unsigned long int ms);
void Stop_Time();
void UARTISR_TX();
void SendBuf_UART(unsigned char *dat,unsigned char len);
void Test_SL015B3();

void main()
{ 
  Init_Hardware();
  g_iTimer=0;
  while(1) Test_SL015B3();
}

//============================================
//  initialize system hardware config
//  timer,uart,port
//============================================
void Init_Hardware()
{
  __disable_interrupt();
  DDRD=0x00;PORTD=0xff;
  TCCR0=0;
  TCNT0=184;
  TIMSK_TOIE0=1;
  TCCR0=0x05;
  UCSRB=0xd8;
  UBRRH=0x00;
  UBRRL=OSC/16/BOUND115200-1;
  g_cUartRxCnt=0;
  g_cUartStatus=UARTSTATUS_FREE;
  __enable_interrupt();
}

//============================================
//  start timer
//============================================
void Start_Time(unsigned long int ms)
{ 
  g_iTimer=ms;
  g_bOverTime=0;
}

//============================================
//  stop timer
//============================================
void Stop_Time()
{ 
  g_iTimer=0;
}

void UARTISR_TX()
{	
  unsigned char tmp;
  if(g_cUartTxCnt==g_cUartTxDataLen)
  {
    if(g_cUartStatus==UARTSTATUS_TX)
    {
      g_cUartStatus=UARTSTATUS_TXSUCC;
      g_cUartRxCnt=0;
      UDR=g_cUartTxCheckSum;
    }
  }
  else
  {
    tmp=g_pUartTxDat[g_cUartTxCnt];
    g_cUartTxCheckSum=g_cUartTxCheckSum^tmp;
    g_cUartTxCnt++;
    UDR=tmp;
  }
}

//============================================
//  send buffer by uart
//============================================
void SendBuf_UART(unsigned char *dat,unsigned char len)
{
  g_cUartStatus=UARTSTATUS_TX;
  g_pUartTxDat=dat;
  g_cUartTxDataLen=len;
  g_cUartTxCnt=0;
  g_cUartTxCheckSum=0;
  UARTISR_TX();
}

//============================================
//  timer0 overflow interrupt
//  10ms
//============================================
#pragma vector = TIMER0_OVF_vect
__interrupt void Timer0_ISR(void)
{
  TCNT0=184;
  if(g_iTimer!=0)
  {
    g_iTimer--;
    if(g_iTimer==0) g_bOverTime=1;
  }
}

//============================================
//  uart rx interrupt
//============================================
#pragma vector = USART_RXC_vect
__interrupt void UART_ISR_RXC(void)
{
  unsigned char CheckSum;
  unsigned char counter;
  g_cUartRxBuf[g_cUartRxCnt]=UDR;
  g_cUartRxCnt++;
  if(g_cUartRxCnt==g_cUartRxBuf[1]+2)
  {
    CheckSum=0;
    for(counter=0;counter<g_cUartRxCnt;counter++) CheckSum=CheckSum^g_cUartRxBuf[counter];
    if(CheckSum==0) g_cUartStatus=UARTSTATUS_RXSUCC;
    else g_cUartStatus=UARTSTATUS_RXERR;
  }
}

//============================================
//  uart tx interrupt
//============================================
#pragma vector = USART_TXC_vect
__interrupt void UART_ISR_TXC(void)
{
  UARTISR_TX();
}

void Test_SL015B3()
{
  #define RCVCMD_SL015B3 g_cUartRxBuf[2]
  #define RCVSTA_SL015B3 g_cUartRxBuf[3]
  g_cCardType=0xff;
  
  if(CARDIN!=0) return;
  
  SendBuf_UART((unsigned char *)(GetCardInformation),sizeof(GetCardInformation)); 
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  Stop_Time();	  
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015B3!=0x31)||(RCVSTA_SL015B3!=0)) return;
  if((g_cUartRxBuf[14]!=0x31) &&(g_cUartRxBuf[14]!=0x32)) return;
  
  SendBuf_UART((unsigned char *)(GetBlcok0SecurityStatus),sizeof(GetBlcok0SecurityStatus));   
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  Stop_Time();	  
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015B3!=0x32)||(RCVSTA_SL015B3!=0)) return;
  if(g_cUartRxBuf[4]!=0) return;
  
  SendBuf_UART((unsigned char *)(WriteBlock0),sizeof(WriteBlock0));   
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  Stop_Time();	  
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015B3!=0x34)||(RCVSTA_SL015B3!=0)) return;
  if(memcmp(&WriteBlock0[4],&g_cUartRxBuf[4],4)!=0) return;
  
  SendBuf_UART((unsigned char *)(ReadBlocks),sizeof(ReadBlocks)); 
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  Stop_Time();	  
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015B3!=0x33)||(RCVSTA_SL015B3!=0)) return;
  if(memcmp(&WriteBlock0[4],&g_cUartRxBuf[4],4)!=0) return;
  
  SendBuf_UART((unsigned char *)(WriteAFI),sizeof(WriteAFI)); 
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  Stop_Time();	  
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015B3!=0x35)||(RCVSTA_SL015B3!=0)) return;
  if(memcmp(&WriteAFI[3],&g_cUartRxBuf[4],1)!=0) return;
  
  SendBuf_UART((unsigned char *)(WriteDSFID),sizeof(WriteDSFID)); 
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  Stop_Time();	  
  if((g_cUartStatus!=UARTSTATUS_RXSUCC)||(RCVCMD_SL015B3!=0x36)||(RCVSTA_SL015B3!=0)) return;
  if(memcmp(&WriteDSFID[3],&g_cUartRxBuf[4],1)!=0) return;
  
  //Glare Red_Led to indicate working ok
  SendBuf_UART((unsigned char *)(TurnOnRed),sizeof(TurnOnRed));
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  Stop_Time();
  Start_Time(30);
  while(g_bOverTime==0);
  Stop_Time();
  
  SendBuf_UART((unsigned char *)(TurnOffRed),sizeof(TurnOffRed));
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));
  Stop_Time();
}
